@@ -1,16 +1,13 @@
-import {
-  flattenPanels,
-  getAffectedPanelIdsForVariable,
-  getAllAffectedPanelIdsForVariableChange,
-  getDependenciesForVariable,
-  getPropsWithVariable,
-} from './utils';
-import { variableAdapters } from '../adapters';
-import { createDataSourceVariableAdapter } from '../datasource/adapter';
-import { createCustomVariableAdapter } from '../custom/adapter';
-import { createQueryVariableAdapter } from '../query/adapter';
 import { PanelModel } from 'app/features/dashboard/state';
 
+import { variableAdapters } from '../adapters';
+import { createCustomVariableAdapter } from '../custom/adapter';
+import { createDataSourceVariableAdapter } from '../datasource/adapter';
+import { createQueryVariableAdapter } from '../query/adapter';
+import { createGraph } from '../state/actions';
+
+import { flattenPanels, getAllAffectedPanelIdsForVariableChange, getPanelVars, getPropsWithVariable } from './utils';
+
 describe('getPropsWithVariable', () => {
   it('when called it should return the correct graph', () => {
     const result = getPropsWithVariable(
@@ -215,46 +212,12 @@ describe('getPropsWithVariable', () => {
   });
 });
 
-describe('getAffectedPanelIdsForVariable', () => {
-  describe('when called with a real world example with rows and repeats', () => {
-    it('then it should return correct panel ids', () => {
-      const panels = dashWithRepeatsAndRows.panels.map(
-        (panel: PanelModel) =>
-          ({
-            id: panel.id,
-            getSaveModel: () => panel,
-          } as unknown as PanelModel)
-      );
-      const result = getAffectedPanelIdsForVariable('query0', panels);
-      expect(result).toEqual([15, 16, 17, 11, 12, 13, 2, 5, 7, 6]);
-    });
-  });
-});
-
 variableAdapters.setInit(() => [
   createDataSourceVariableAdapter(),
   createCustomVariableAdapter(),
   createQueryVariableAdapter(),
 ]);
 
-describe('getDependenciesForVariable', () => {
-  describe('when called with a real world example with dependencies', () => {
-    it('then it should return correct dependencies', () => {
-      const {
-        templating: { list: variables },
-      } = dashWithTemplateDependenciesAndPanels;
-      const result = getDependenciesForVariable('ds_instance', variables, new Set());
-      expect([...result]).toEqual([
-        'ds',
-        'query_with_ds',
-        'depends_on_query_with_ds',
-        'depends_on_query_with_ds_regex',
-        'depends_on_all',
-      ]);
-    });
-  });
-});
-
 describe('getAllAffectedPanelIdsForVariableChange ', () => {
   describe('when called with a real world example with dependencies and panels', () => {
     it('then it should return correct panelIds', () => {
@@ -262,15 +225,11 @@ describe('getAllAffectedPanelIdsForVariableChange ', () => {
         panels: panelsAsJson,
         templating: { list: variables },
       } = dashWithTemplateDependenciesAndPanels;
-      const panels = panelsAsJson.map(
-        (panel: PanelModel) =>
-          ({
-            id: panel.id,
-            getSaveModel: () => panel,
-          } as unknown as PanelModel)
-      );
-      const result = getAllAffectedPanelIdsForVariableChange('ds_instance', variables, panels);
-      expect(result).toEqual([2, 3, 4, 5]);
+      const panelVarPairs = getPanelVars(panelsAsJson);
+      const varGraph = createGraph(variables);
+
+      const result = [...getAllAffectedPanelIdsForVariableChange(['ds_instance'], varGraph, panelVarPairs)];
+      expect(result).toEqual([5, 2, 4, 3]);
     });
   });
 
@@ -280,14 +239,9 @@ describe('getAllAffectedPanelIdsForVariableChange ', () => {
         panels: panelsAsJson,
         templating: { list: variables },
       } = dashWithTemplateDependenciesAndPanels;
-      const panels = panelsAsJson.map(
-        (panel: PanelModel) =>
-          ({
-            id: panel.id,
-            getSaveModel: () => panel,
-          } as unknown as PanelModel)
-      );
-      const result = getAllAffectedPanelIdsForVariableChange('depends_on_all', variables, panels);
+      const panelVarPairs = getPanelVars(panelsAsJson);
+      const varGraph = createGraph(variables);
+      const result = [...getAllAffectedPanelIdsForVariableChange(['depends_on_all'], varGraph, panelVarPairs)];
       expect(result).toEqual([2]);
     });
   });
@@ -298,14 +252,9 @@ describe('getAllAffectedPanelIdsForVariableChange ', () => {
         panels: panelsAsJson,
         templating: { list: variables },
       } = dashWithAllVariables;
-      const panels = panelsAsJson.map(
-        (panel: PanelModel) =>
-          ({
-            id: panel.id,
-            getSaveModel: () => panel,
-          } as unknown as PanelModel)
-      );
-      const result = getAllAffectedPanelIdsForVariableChange('unknown', variables, panels);
+      const panelVarPairs = getPanelVars(panelsAsJson);
+      const varGraph = createGraph(variables);
+      const result = [...getAllAffectedPanelIdsForVariableChange(['unknown'], varGraph, panelVarPairs)];
       expect(result).toEqual([2, 3]);
     });
   });
@@ -1541,8 +1490,8 @@ const dashWithTemplateDependenciesAndPanels: any = {
       {
         current: {
           selected: false,
-          text: 'TestData DB',
-          value: 'TestData DB',
+          text: 'TestData',
+          value: 'TestData',
         },
         description: null,
         error: null,
@@ -1675,8 +1624,8 @@ const dashWithTemplateDependenciesAndPanels: any = {
         allValue: null,
         current: {
           selected: true,
-          text: 'TestData DB',
-          value: 'TestData DB',
+          text: 'TestData',
+          value: 'TestData',
         },
         description: null,
         error: null,
@@ -1688,8 +1637,8 @@ const dashWithTemplateDependenciesAndPanels: any = {
         options: [
           {
             selected: true,
-            text: 'TestData DB',
-            value: 'TestData DB',
+            text: 'TestData',
+            value: 'TestData',
           },
           {
             selected: false,
@@ -1697,7 +1646,7 @@ const dashWithTemplateDependenciesAndPanels: any = {
             value: 'gdev-testdata',
           },
         ],
-        query: 'TestData DB, gdev-testdata',
+        query: 'TestData, gdev-testdata',
         queryValue: '',
         skipUrlSync: false,
         type: 'custom',
